/*******************************************************************************
  MPLAB Harmony Application Source File

  Company:
    Microchip Technology Inc.

  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It
    implements the logic of the application's state machine and it may call
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "app.h"
#include "peripheral/gpio/plib_gpio.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.

    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

/* TODO:  Add any necessary callback functions.
*/

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************


/* TODO:  Add any necessary local functions.
*/


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */
volatile bool flag = false;

// Callback funkcija za tajmerski prekid na TMR2 modulu:
void TMR3_IntHandler(uint32_t status, uintptr_t context){
    flag = true;
    //printf("TMR3 ISR \r\n");
}

void APP_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;
    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
}


/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

#define SDCARD_MOUNT_NAME    "/mnt/MMC"
#define SDCARD_DEV_NAME      "/dev/mmcblka1"

SYS_FS_HANDLE               fileHandle;
//static uint8_t  readWriteBuffer[512];

MS8607_sens_t senzor;

void APP_Tasks ( void )
{  


    /* Check the application's current state. */
    switch ( appData.state )
    {
        /* Application's initial state. */
        case APP_STATE_INIT:                                                    // Stanje za inicijalizaciju sistema.
        {
            bool appInitialized = true;

            if (appInitialized)
            {
                printf("INIT DONE\r\n");
                appData.state = APP_MOUNT_DISK;                                 // Ukoliko je inicijalizacija prosla potrebno je preci na sledeci korak - povezivanje File System-a i kartice.
            }
            break;
        }

        // Povezivanje File System-a i kartice:
        case APP_MOUNT_DISK:
        {
            if(SYS_FS_Mount(SDCARD_DEV_NAME, SDCARD_MOUNT_NAME, FAT, 0, NULL)!=0){      // Ukoliko povezivanje nije uspelo:
                // FAT - File Allocation Table (tip file sistema).
                // Funkcija SYS_FS_Mount vraca 0 ukoliko je povezivanje uspesno.  
                appData.state = APP_MOUNT_DISK;                                 // Potrebno je pokusati ponovo - povratak na isto stanje APP_MOUNT_DISK.
            }
            else{
                printf("MOUNT OK\r\n");                                         
                appData.state = APP_SET_CURRENT_DRIVE;                          // Ukoliko je povezivanje uspelo potrebno je preci na sledeci korak - postavljanje kartice kao trenutnog drive-a sistema. 
            }
            break;
        }
        
        // Prekid veze File System-a i kartice:
        case APP_UNMOUNT_DISK:
        {
             if(SYS_FS_Unmount(SDCARD_MOUNT_NAME) != 0){                        // Ukoliko prekid veze nije uspeo (SYS_FS_Unmount vraca 0 ukoliko je prekid veze uspesan):
                printf("UNMOUNT ERROR\r\n");
                appData.state = APP_UNMOUNT_DISK;
             }
             else{
                 printf("UNMOUNT OK\r\n");
                 appData.state = APP_SET_CURRENT_DRIVE;                         // Ukoliko je prekid veze uspeo potrebno je preci na sledeci korak - postavljanje kartice kao trenutnog drive-a sistema. 
             }
             break;
        }

        // Postavljanje kartice kao trenutnog drive-a sistema:
        case APP_SET_CURRENT_DRIVE: 
        {
            if(SYS_FS_CurrentDriveSet(SDCARD_MOUNT_NAME) == SYS_FS_RES_FAILURE) // Postavljanje kartice kao trenutnog drive-a sistema.      
            // SYS_FS_CurrentDriveSet vraca SYS_FS_RES_SUCCESS (0) za Operation succeeded ili SYS_FS_RES_FAILURE (1) Operation failed.
            {
                printf("SET CURRENT DRIVE ERROR\r\n");
                appData.state = APP_ERROR;                                      // Ukoliko postavljanje kartice kao trenutnog drive-a sistema nije uspelo potrebno je preci na stanje pojave greske.
            }
            else
            {
                printf("SET CURRENT DRIVE OK\r\n");
                appData.state = APP_OPEN_FILE;                                  // Ukoliko je postavljanje kartice kao trenutnog drive-a uspelo potrebno je preci na sledeci korak - prikupljanje podataka sa senzora.
            }
            break;
        
        }
        
         // Otvaranje .txt datoteke:
        case APP_OPEN_FILE:
        {
            fileHandle = SYS_FS_FileOpen("pht.txt", (SYS_FS_FILE_OPEN_APPEND));      // SYS_FS_FileOpen funkcija otvara .txt datoteku sa prosledjenim nazivom - "pht.txt".
            // Drugi prolsedjeni parametar SYS_FS_FileOpen funkcije oznacava mod otvaranja datoteke - u ovom slucaju SYS_FS_FILE_OPEN_WRITE_PLUS omogucava citanje datoteke ukoliko ona postoji, upis u postojecu datoteku ili kreiranje nove datoteke i upis u nju.
            
            if(fileHandle == SYS_FS_HANDLE_INVALID)                             // Ukoliko otvaranje datoteke nije uspelo kao fileHandle funkcija SYS_FS_FileOpen vraca SYS_FS_HANDLE_INVALID.
            {
                appData.state = APP_ERROR;                                      // Ukoliko otvaranje datoteke nije uslepo potrebno je preci u stanje za pojavu greske.
            }
            else
            {
                appData.state = APP_WAIT_FOR_DATA;                              // Ukoliko je dadoteka otvorena u SYS_FS_FILE_OPEN_WRITE_PLUS modu prelazi se na sledeci korak - upis podataka u datoteku.      
            }
            break;
        }
        
        // Cekanje podataka sa MS8607 senzora:
        case APP_WAIT_FOR_DATA:
        {
            
            if(flag)                                                            // Ukoliko je generisan interrupt na TMR2 (proslo je 500 ms).
            {
                flag = false;                                                   // Resetovanje flag-a.
                
                // Ocitavanje podataka o temperaturi, pritisku i vlaznosti vazduha sa MS8607 senzora:
                MS8607_read_temperature(&senzor, MS8607_PT_D2_CONVERSION_OSR2048);
                MS8607_read_pressure(&senzor, MS8607_PT_D1_CONVERSION_OSR2048);
                MS8607_read_relative_humidity(&senzor, MS8607_H_OSR_2048, MS8607_H_MEASURE_HOLD_MASTER);
                
                
                char pht_data[27];                                              // Niz podataka za upis.
            
                sprintf(pht_data,"T:%.2f|P:%.2f|H:%.2f\r\n", senzor.temperature/100.0, senzor.pressure/100.0, senzor.relative_humidity/100.0);      // Formatiranje podataka o pritisku, temperaturi i vlaznosti u jedan string (pht_data niz) koji ce biti upisan u .txt.
                
                printf("%s\r\n", pht_data);
                
                uint32_t bytesWritten = SYS_FS_FileWrite(fileHandle, pht_data, strlen(pht_data));   //  Funkciji SYS_FS_FileWrite prosledjuju se fileHandle, niz sa podacima za upis i kolicina podataka za upis.
            
                
                if(GPIO_PinRead(GPIO_PIN_RG0)==0)                               // Ukoliko je pritisnut taster na pinu RG0 potrebno je prekinuti upis podataka i zatvoriti fajl. (Napomena: RG0 - pull-up+button press level low) 
                {
                    appData.state = APP_CLOSE_FILE;
                }
                else{
                    appData.state = APP_WAIT_FOR_DATA;                          // Ukoliko nije pritisnut taster RG0 nastavlja se sa cekanjem na podatke.
                }
            }
            else{
                
                appData.state = APP_WAIT_FOR_DATA;                              // Ukoliko nije proslo 500 ms potrebno je ostati u datom stanju - cekanja na ocitavanje podataka sa senzora.
            }
            break;
        }
        
        
        // Zatvaranje datoteke:
        case APP_CLOSE_FILE:
        {
            SYS_FS_FileClose(fileHandle);
            
            appData.state = APP_IDLE;
            break;
        }
        
        // Ukoliko je doslo do greske:
        case APP_ERROR:
        {
            printf("APP ERROR\n");
            CORETIMER_DelayMs(1000);
            break;
        }
        
        // Neaktivno stanje - potrebno je resetovati sistem i izvrsavati kod ponovo:
        case APP_IDLE:
        {
            printf("APP IDLE --> RESET\n");
            CORETIMER_DelayMs(10000);
            break;
        }
        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}


/*******************************************************************************
 End of File
 */
