/*******************************************************************************
  Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This file contains the "main" function for a project.

  Description:
    This file contains the "main" function for a project.  The
    "main" function calls the "SYS_Initialize" function to initialize the state
    machines of all modules in the system
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include "definitions.h"                // SYS function prototypes

// *****************************************************************************
// *****************************************************************************
// Section: Main Entry Point
// *****************************************************************************
// *****************************************************************************
int main ( void )
{
    
    /* Initialize all modules */
    SYS_Initialize ( NULL );

    TMR3_CallbackRegister(TMR3_IntHandler, (uintptr_t)NULL);                    // Registrovanje callback funkcije za interrupt sa TMR3.
    TMR3_Start();                                                               // Startovanje TMR3.
    
    MS8607_reset_PTsens();                                                      // Resetovanje senzora pritiska i temperature.
    CORETIMER_DelayMs(1000);
    MS8607_reset_Hsens();                                                       // Resetovanje senzora vlaznosti vazduha.
    CORETIMER_DelayMs(1000);
    MS8607_read_PT_calibration_data(&senzor);                                   // Ocitavanje kalibracionih koeficijenata i njihov upis na senzor strukturu.
    
    // Ispis kalibracionih koeficijenata:
    printf("P sens: %u\r\n", senzor.pressure_sensitivity);
    printf("P offset: %u\r\n", senzor.pressure_offset);
    printf("P sens temp coeff: %u\r\n", senzor.temp_coeff_pressure_sensitivity);
    printf("P offset temp coeff: %u\r\n", senzor.temp_coeff_pressure_offset);
    printf("T temp coeff: %u\r\n", senzor.temp_coeff_temp);
    printf("T ref: %u\r\n", senzor.ref_temp);
    printf("___________________________________\r\n");
    CORETIMER_DelayMs(1000);

    
    while ( true )
    {
        SYS_Tasks ( );
    }

    /* Execution should not come here during normal operation */

    return ( EXIT_FAILURE );
}


/*******************************************************************************
 End of File
*/

